@@ -1,7 +1,7 @@
 """
- This module contains all of the GEOS ctypes function prototypes. Each
- prototype handles the interaction between the GEOS library and Python
- via ctypes.
+This module contains all of the GEOS ctypes function prototypes. Each
+prototype handles the interaction between the GEOS library and Python
+via ctypes.
 """
 
 from django.contrib.gis.geos.prototypes.coordseq import (  # NOQA
@@ -51,6 +51,8 @@ from django.contrib.gis.geos.prototypes.predicates import (  # NOQA
     geos_disjoint,
     geos_equals,
     geos_equalsexact,
+    geos_equalsidentical,
+    geos_hasm,
     geos_hasz,
     geos_intersects,
     geos_isclosed,
