@@ -1,8 +1,8 @@
 """Default variable filters."""
+
 import random as random_module
 import re
 import types
-import warnings
 from decimal import ROUND_HALF_UP, Context, Decimal, InvalidOperation, getcontext
 from functools import wraps
 from inspect import unwrap
@@ -12,7 +12,6 @@ from urllib.parse import quote
 
 from django.utils import formats
 from django.utils.dateformat import format, time_format
-from django.utils.deprecation import RemovedInDjango51Warning
 from django.utils.encoding import iri_to_uri
 from django.utils.html import avoid_wrapping, conditional_escape, escape, escapejs
 from django.utils.html import json_script as _json_script
@@ -170,9 +169,10 @@ def floatformat(text, arg=-1):
         # Exponent values can be "F", "n", "N".
         number_of_digits_and_exponent_sum = 0
 
-    # Values with more than 200 digits, or with a large exponent, are returned "as is"
-    # to avoid high memory consumption and potential denial-of-service attacks.
-    # The cut-off of 200 is consistent with django.utils.numberformat.floatformat().
+    # Values with more than 200 digits, or with a large exponent, are returned
+    # "as is" to avoid high memory consumption and potential denial-of-service
+    # attacks. The cut-off of 200 is consistent with
+    # django.utils.numberformat.floatformat().
     if number_of_digits_and_exponent_sum > 200:
         return input_val
 
@@ -282,7 +282,8 @@ def stringformat(value, arg):
     This specifier uses Python string formatting syntax, with the exception
     that the leading "%" is dropped.
 
-    See https://docs.python.org/library/stdtypes.html#printf-style-string-formatting
+    See
+    https://docs.python.org/library/stdtypes.html#printf-style-string-formatting
     for documentation of Python string formatting.
     """
     if isinstance(value, tuple):
@@ -431,7 +432,10 @@ def rjust(value, arg):
 @stringfilter
 def center(value, arg):
     """Center the value in a field of a given width."""
-    return value.center(int(arg))
+    width = int(arg)
+    if width <= 0:
+        return value
+    return f"{value:^{width}}"
 
 
 @register.filter
@@ -457,6 +461,16 @@ def escape_filter(value):
     return conditional_escape(value)
 
 
+@register.filter(is_safe=True)
+def escapeseq(value):
+    """
+    An "escape" filter for sequences. Mark each element in the sequence,
+    individually, as a string that should be auto-escaped. Return a list with
+    the results.
+    """
+    return [conditional_escape(obj) for obj in value]
+
+
 @register.filter(is_safe=True)
 @stringfilter
 def force_escape(value):
@@ -599,8 +613,9 @@ def join(value, arg, autoescape=True):
     """Join a list with a string, like Python's ``str.join(list)``."""
     try:
         if autoescape:
-            value = [conditional_escape(v) for v in value]
-        data = conditional_escape(arg).join(value)
+            data = conditional_escape(arg).join([conditional_escape(v) for v in value])
+        else:
+            data = arg.join(value)
     except TypeError:  # Fail silently if arg isn't iterable.
         return value
     return mark_safe(data)
@@ -624,24 +639,13 @@ def length(value):
         return 0
 
 
-@register.filter(is_safe=False)
-def length_is(value, arg):
-    """Return a boolean of whether the value's length is the argument."""
-    warnings.warn(
-        "The length_is template filter is deprecated in favor of the length template "
-        "filter and the == operator within an {% if %} tag.",
-        RemovedInDjango51Warning,
-    )
-    try:
-        return len(value) == int(arg)
-    except (ValueError, TypeError):
-        return ""
-
-
 @register.filter(is_safe=True)
 def random(value):
     """Return a random item from the list."""
-    return random_module.choice(value)
+    try:
+        return random_module.choice(value)
+    except IndexError:
+        return ""
 
 
 @register.filter("slice", is_safe=True)
@@ -658,7 +662,7 @@ def slice_filter(value, arg):
                 bits.append(int(x))
         return value[slice(*bits)]
 
-    except (ValueError, TypeError):
+    except (ValueError, TypeError, KeyError):
         return value  # Fail silently.
 
 
