@@ -4,6 +4,7 @@ import re
 import zoneinfo
 from datetime import datetime, timedelta
 from importlib import import_module
+from pathlib import Path
 from unittest import skipUnless
 
 from django import forms
@@ -22,12 +23,11 @@ from django.db.models import (
     ManyToManyField,
     UUIDField,
 )
-from django.test import SimpleTestCase, TestCase, ignore_warnings, override_settings
+from django.test import SimpleTestCase, TestCase, override_settings
 from django.test.selenium import screenshot_cases
 from django.test.utils import requires_tz_support
 from django.urls import reverse
 from django.utils import translation
-from django.utils.deprecation import RemovedInDjango60Warning
 
 from .models import (
     Advisor,
@@ -109,7 +109,6 @@ class AdminFormfieldForDBFieldTests(SimpleTestCase):
     def test_TextField(self):
         self.assertFormfield(Event, "description", widgets.AdminTextareaWidget)
 
-    @ignore_warnings(category=RemovedInDjango60Warning)
     def test_URLField(self):
         self.assertFormfield(Event, "link", widgets.AdminURLFieldWidget)
 
@@ -301,7 +300,8 @@ class AdminFormfieldForDBFieldTests(SimpleTestCase):
 class AdminFormfieldForDBFieldWithRequestTests(TestDataMixin, TestCase):
     def test_filter_choices_by_request_user(self):
         """
-        Ensure the user can only see their own cars in the foreign key dropdown.
+        Ensure the user can only see their own cars in the foreign key
+        dropdown.
         """
         self.client.force_login(self.superuser)
         response = self.client.get(reverse("admin:admin_widgets_cartire_add"))
@@ -324,7 +324,6 @@ class AdminForeignKeyRawIdWidget(TestDataMixin, TestCase):
     def setUp(self):
         self.client.force_login(self.superuser)
 
-    @ignore_warnings(category=RemovedInDjango60Warning)
     def test_nonexistent_target_id(self):
         band = Band.objects.create(name="Bogey Blues")
         pk = band.pk
@@ -340,7 +339,6 @@ class AdminForeignKeyRawIdWidget(TestDataMixin, TestCase):
             "Select a valid choice. That choice is not one of the available choices.",
         )
 
-    @ignore_warnings(category=RemovedInDjango60Warning)
     def test_invalid_target_id(self):
         for test_str in ("Iñtërnâtiônàlizætiøn", "1234'", -1234):
             # This should result in an error message, not a server exception.
@@ -401,7 +399,8 @@ class AdminDateWidgetTest(SimpleTestCase):
         self.assertHTMLEqual(
             w.render("test", datetime(2007, 12, 1, 9, 30)),
             '<p class="date">'
-            '<input value="2007-12-01" type="text" class="vDateField" name="test" '
+            '<input aria-describedby="id_test_timezone_warning_helptext" '
+            'value="2007-12-01" type="text" class="vDateField" name="test" '
             'size="10"></p>',
         )
         # pass attrs to widget
@@ -409,7 +408,8 @@ class AdminDateWidgetTest(SimpleTestCase):
         self.assertHTMLEqual(
             w.render("test", datetime(2007, 12, 1, 9, 30)),
             '<p class="date">'
-            '<input value="2007-12-01" type="text" class="myDateField" name="test" '
+            '<input aria-describedby="id_test_timezone_warning_helptext" '
+            'value="2007-12-01" type="text" class="myDateField" name="test" '
             'size="20"></p>',
         )
 
@@ -420,7 +420,8 @@ class AdminTimeWidgetTest(SimpleTestCase):
         self.assertHTMLEqual(
             w.render("test", datetime(2007, 12, 1, 9, 30)),
             '<p class="time">'
-            '<input value="09:30:00" type="text" class="vTimeField" name="test" '
+            '<input aria-describedby="id_test_timezone_warning_helptext" '
+            'value="09:30:00" type="text" class="vTimeField" name="test" '
             'size="8"></p>',
         )
         # pass attrs to widget
@@ -428,7 +429,8 @@ class AdminTimeWidgetTest(SimpleTestCase):
         self.assertHTMLEqual(
             w.render("test", datetime(2007, 12, 1, 9, 30)),
             '<p class="time">'
-            '<input value="09:30:00" type="text" class="myTimeField" name="test" '
+            '<input aria-describedby="id_test_timezone_warning_helptext" '
+            'value="09:30:00" type="text" class="myTimeField" name="test" '
             'size="20"></p>',
         )
 
@@ -437,12 +439,16 @@ class AdminSplitDateTimeWidgetTest(SimpleTestCase):
     def test_render(self):
         w = widgets.AdminSplitDateTime()
         self.assertHTMLEqual(
-            w.render("test", datetime(2007, 12, 1, 9, 30)),
+            w.render("test", datetime(2007, 12, 1, 9, 30), attrs={"id": "id_test"}),
             '<p class="datetime">'
-            'Date: <input value="2007-12-01" type="text" class="vDateField" '
-            'name="test_0" size="10"><br>'
-            'Time: <input value="09:30:00" type="text" class="vTimeField" '
-            'name="test_1" size="8"></p>',
+            '<label for="id_test_0">Date:</label> '
+            '<input aria-describedby="id_test_timezone_warning_helptext" '
+            'value="2007-12-01" type="text" class="vDateField" '
+            'name="test_0" size="10" id="id_test_0"><br>'
+            '<label for="id_test_1">Time:</label> '
+            '<input aria-describedby="id_test_timezone_warning_helptext" '
+            'value="09:30:00" type="text" class="vTimeField" '
+            'name="test_1" size="8" id="id_test_1"></p>',
         )
 
     def test_localization(self):
@@ -451,12 +457,16 @@ class AdminSplitDateTimeWidgetTest(SimpleTestCase):
         with translation.override("de-at"):
             w.is_localized = True
             self.assertHTMLEqual(
-                w.render("test", datetime(2007, 12, 1, 9, 30)),
+                w.render("test", datetime(2007, 12, 1, 9, 30), attrs={"id": "id_test"}),
                 '<p class="datetime">'
-                'Datum: <input value="01.12.2007" type="text" '
-                'class="vDateField" name="test_0"size="10"><br>'
-                'Zeit: <input value="09:30:00" type="text" class="vTimeField" '
-                'name="test_1" size="8"></p>',
+                '<label for="id_test_0">Datum:</label> '
+                '<input aria-describedby="id_test_timezone_warning_helptext" '
+                'value="01.12.2007" type="text" '
+                'class="vDateField" name="test_0" size="10" id="id_test_0"><br>'
+                '<label for="id_test_1">Zeit:</label> '
+                '<input aria-describedby="id_test_timezone_warning_helptext" '
+                'value="09:30:00" type="text" class="vTimeField" '
+                'name="test_1" size="8" id="id_test_1"></p>',
             )
 
 
@@ -968,6 +978,21 @@ class RelatedFieldWidgetWrapperTests(SimpleTestCase):
         """
         self.assertHTMLEqual(output, expected)
 
+    def test_non_select_widget_cant_change_delete_related(self):
+        main_band = Event._meta.get_field("main_band")
+        widget = widgets.AdminRadioSelect()
+        wrapper = widgets.RelatedFieldWidgetWrapper(
+            widget,
+            main_band,
+            widget_admin_site,
+            can_add_related=True,
+            can_change_related=True,
+            can_delete_related=True,
+        )
+        self.assertTrue(wrapper.can_add_related)
+        self.assertFalse(wrapper.can_change_related)
+        self.assertFalse(wrapper.can_delete_related)
+
 
 @override_settings(ROOT_URLCONF="admin_widgets.urls")
 class AdminWidgetSeleniumTestCase(AdminSeleniumTestCase):
@@ -1055,8 +1080,8 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
 
     def test_calendar_nonday_class(self):
         """
-        Ensure cells that are not days of the month have the `nonday` CSS class.
-        Refs #4574.
+        Ensure cells that are not days of the month have the `nonday` CSS
+        class. Refs #4574.
         """
         from selenium.webdriver.common.by import By
 
@@ -1187,9 +1212,9 @@ class DateTimePickerShortcutsSeleniumTests(AdminWidgetSeleniumTestCase):
         date/time/datetime picker shortcuts work in the current time zone.
         Refs #20663.
 
-        This test case is fairly tricky, it relies on selenium still running the browser
-        in the default time zone "America/Chicago" despite `override_settings` changing
-        the time zone to "Asia/Singapore".
+        This test case is fairly tricky, it relies on selenium still running
+        the browser in the default time zone "America/Chicago" despite
+        `override_settings` changing the time zone to "Asia/Singapore".
         """
         from selenium.webdriver.common.by import By
 
@@ -1505,7 +1530,8 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         self.select_option(from_box, str(self.peter.id))
         self.select_option(from_box, str(self.lisa.id))
 
-        # Confirm they're selected after clicking inactive buttons: ticket #26575
+        # Confirm they're selected after clicking inactive buttons: ticket
+        # #26575
         self.assertSelectedOptions(from_box, [str(self.peter.id), str(self.lisa.id)])
         self.selenium.find_element(By.ID, remove_button).click()
         self.assertSelectedOptions(from_box, [str(self.peter.id), str(self.lisa.id)])
@@ -1518,7 +1544,8 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         self.select_option(to_box, str(self.jason.id))
         self.select_option(to_box, str(self.john.id))
 
-        # Confirm they're selected after clicking inactive buttons: ticket #26575
+        # Confirm they're selected after clicking inactive buttons: ticket
+        # #26575
         self.assertSelectedOptions(to_box, [str(self.jason.id), str(self.john.id)])
         self.selenium.find_element(By.ID, choose_button).click()
         self.assertSelectedOptions(to_box, [str(self.jason.id), str(self.john.id)])
@@ -1544,6 +1571,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
             )
 
             self.wait_page_ready()
+            self.trigger_resize()
             self.execute_basic_operations("vertical", "students")
             self.execute_basic_operations("horizontal", "alumni")
 
@@ -1783,12 +1811,11 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         )
 
 
-@ignore_warnings(category=RemovedInDjango60Warning)
 class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
     def setUp(self):
         super().setUp()
-        Band.objects.create(id=42, name="Bogey Blues")
-        Band.objects.create(id=98, name="Green Potatoes")
+        self.blues = Band.objects.create(name="Bogey Blues")
+        self.potatoes = Band.objects.create(name="Green Potatoes")
 
     @screenshot_cases(["desktop_size", "mobile_size", "rtl", "dark", "high_contrast"])
     def test_ForeignKey(self):
@@ -1810,23 +1837,23 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
         self.selenium.find_element(By.ID, "lookup_id_main_band").click()
         self.wait_for_and_switch_to_popup()
         link = self.selenium.find_element(By.LINK_TEXT, "Bogey Blues")
-        self.assertIn("/band/42/", link.get_attribute("href"))
+        self.assertIn(f"/band/{self.blues.pk}/", link.get_attribute("href"))
         link.click()
 
         # The field now contains the selected band's id
         self.selenium.switch_to.window(main_window)
-        self.wait_for_value("#id_main_band", "42")
+        self.wait_for_value("#id_main_band", str(self.blues.pk))
 
         # Reopen the popup window and click on another band
         self.selenium.find_element(By.ID, "lookup_id_main_band").click()
         self.wait_for_and_switch_to_popup()
         link = self.selenium.find_element(By.LINK_TEXT, "Green Potatoes")
-        self.assertIn("/band/98/", link.get_attribute("href"))
+        self.assertIn(f"/band/{self.potatoes.pk}/", link.get_attribute("href"))
         link.click()
 
         # The field now contains the other selected band's id
         self.selenium.switch_to.window(main_window)
-        self.wait_for_value("#id_main_band", "98")
+        self.wait_for_value("#id_main_band", str(self.potatoes.pk))
 
     def test_many_to_many(self):
         from selenium.webdriver.common.by import By
@@ -1857,23 +1884,25 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
         self.selenium.find_element(By.ID, "lookup_id_supporting_bands").click()
         self.wait_for_and_switch_to_popup()
         link = self.selenium.find_element(By.LINK_TEXT, "Bogey Blues")
-        self.assertIn("/band/42/", link.get_attribute("href"))
+        self.assertIn(f"/band/{self.blues.pk}/", link.get_attribute("href"))
         link.click()
 
         # The field now contains the selected band's id
         self.selenium.switch_to.window(main_window)
-        self.wait_for_value("#id_supporting_bands", "42")
+        self.wait_for_value("#id_supporting_bands", str(self.blues.pk))
 
         # Reopen the popup window and click on another band
         self.selenium.find_element(By.ID, "lookup_id_supporting_bands").click()
         self.wait_for_and_switch_to_popup()
         link = self.selenium.find_element(By.LINK_TEXT, "Green Potatoes")
-        self.assertIn("/band/98/", link.get_attribute("href"))
+        self.assertIn(f"/band/{self.potatoes.pk}/", link.get_attribute("href"))
         link.click()
 
         # The field now contains the two selected bands' ids
         self.selenium.switch_to.window(main_window)
-        self.wait_for_value("#id_supporting_bands", "42,98")
+        self.wait_for_value(
+            "#id_supporting_bands", f"{self.blues.pk},{self.potatoes.pk}"
+        )
 
 
 class RelatedFieldWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
@@ -1953,7 +1982,7 @@ class RelatedFieldWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
 class ImageFieldWidgetsSeleniumTests(AdminWidgetSeleniumTestCase):
     name_input_id = "id_name"
     photo_input_id = "id_photo"
-    tests_files_folder = "%s/files" % os.getcwd()
+    tests_files_folder = "%s/files" % Path(__file__).parent.parent
     clear_checkbox_id = "photo-clear_id"
 
     def _submit_and_wait(self):
