@@ -43,3 +43,10 @@ class Store(models.Model):
 
     def __str__(self):
         return self.name
+
+
+class Employee(models.Model):
+    work_day_preferences = models.JSONField()
+
+    class Meta:
+        required_db_features = {"supports_json_field"}
