@@ -34,6 +34,7 @@ from .models import (
     RChild,
     RChildChild,
     Referrer,
+    RProxy,
     S,
     T,
     User,
@@ -493,7 +494,7 @@ class DeletionTests(TestCase):
         # and there are no more cascades.
         # 1 query to find the users for the avatar.
         # 1 query to delete the user
-        # 1 query to null out user.avatar, because we can't defer the constraint
+        # 1 query to null out user.avatar, since we can't defer the constraint
         # 1 query to delete the avatar
         self.assertNumQueries(4, a.delete)
         self.assertFalse(User.objects.exists())
@@ -517,8 +518,8 @@ class DeletionTests(TestCase):
         batch_size = connection.ops.bulk_batch_size(["pk"], objs)
         # The related fetches are done in batches.
         batches = ceil(len(objs) / batch_size)
-        # One query for Avatar.objects.all() and then one related fast delete for
-        # each batch.
+        # One query for Avatar.objects.all() and then one related fast delete
+        # for each batch.
         fetches_to_mem = 1 + batches
         # The Avatar objects are going to be deleted in batches of
         # GET_ITERATOR_CHUNK_SIZE.
@@ -536,7 +537,7 @@ class DeletionTests(TestCase):
 
         # TEST_SIZE / batch_size (select related `T` instances)
         # + 1 (select related `U` instances)
-        # + TEST_SIZE / GET_ITERATOR_CHUNK_SIZE (delete `T` instances in batches)
+        # + TEST_SIZE / GET_ITERATOR_CHUNK_SIZE (delete `T` matches in batches)
         # + 1 (delete `s`)
         expected_num_queries = ceil(TEST_SIZE / batch_size)
         expected_num_queries += ceil(TEST_SIZE / GET_ITERATOR_CHUNK_SIZE) + 2
@@ -675,6 +676,14 @@ class DeletionTests(TestCase):
                 )
                 signal.disconnect(receiver, sender=Referrer)
 
+    def test_keep_parents_does_not_delete_proxy_related(self):
+        r_child = RChild.objects.create()
+        r_proxy = RProxy.objects.get(pk=r_child.pk)
+        Origin.objects.create(r_proxy=r_proxy)
+        self.assertEqual(Origin.objects.count(), 1)
+        r_child.delete(keep_parents=True)
+        self.assertEqual(Origin.objects.count(), 1)
+
 
 class FastDeleteTests(TestCase):
     def test_fast_delete_all(self):
