@@ -4,5 +4,3 @@ except ImportError:
     from django.contrib import admin
 
     admin.GISModelAdmin = admin.ModelAdmin
-    # RemovedInDjango50Warning.
-    admin.OSMGeoAdmin = admin.ModelAdmin
