@@ -196,8 +196,8 @@ class HumanizeTests(SimpleTestCase):
             None,
             "1,234,567",
             "-1,234,567",
-            "１,２３４,５６７.１２",
-            "-１,２３４,５６７.１２",
+            "1,234,567.12",
+            "-1,234,567.12",
             "the quick brown fox jumped over the lazy dog",
         )
         with translation.override("en"):
@@ -238,7 +238,7 @@ class HumanizeTests(SimpleTestCase):
             "-１２３４５６７.１２",
             "the quick brown fox jumped over the lazy dog",
         )
-        result_list = (
+        result_list_en = (
             "100",
             "-100",
             "1,000",
@@ -268,13 +268,49 @@ class HumanizeTests(SimpleTestCase):
             None,
             "1,234,567",
             "-1,234,567",
-            "１,２３４,５６７.１２",
-            "-１,２３４,５６７.１２",
+            "1,234,567.12",
+            "-1,234,567.12",
+            "the quick brown fox jumped over the lazy dog",
+        )
+        result_list_de = (
+            "100",
+            "-100",
+            "1.000",
+            "-1.000",
+            "10.123",
+            "-10.123",
+            "10.311",
+            "-10.311",
+            "1.000.000",
+            "-1.000.000",
+            "1.234.567,25",
+            "-1.234.567,25",
+            "100",
+            "-100",
+            "1.000",
+            "-1.000",
+            "10.123",
+            "-10.123",
+            "10.311",
+            "-10.311",
+            "1.000.000",
+            "-1.000.000",
+            "1.234.567,1234567",
+            "-1.234.567,1234567",
+            "1.234.567,1234567",
+            "-1.234.567,1234567",
+            None,
+            "1.234.567",
+            "-1.234.567",
+            "1.234.567,12",
+            "-1.234.567,12",
             "the quick brown fox jumped over the lazy dog",
         )
         with self.settings(USE_THOUSAND_SEPARATOR=False):
             with translation.override("en"):
-                self.humanize_tester(test_list, result_list, "intcomma")
+                self.humanize_tester(test_list, result_list_en, "intcomma")
+            with translation.override("de"):
+                self.humanize_tester(test_list, result_list_de, "intcomma")
 
     def test_intcomma_without_number_grouping(self):
         # Regression for #17414
@@ -441,7 +477,7 @@ class HumanizeTests(SimpleTestCase):
     def test_naturalday_uses_localtime(self):
         # Regression for #18504
         # This is 2012-03-08HT19:30:00-06:00 in America/Chicago
-        dt = datetime.datetime(2012, 3, 9, 1, 30, tzinfo=datetime.timezone.utc)
+        dt = datetime.datetime(2012, 3, 9, 1, 30, tzinfo=datetime.UTC)
 
         orig_humanize_datetime, humanize.datetime = humanize.datetime, MockDateTime
         try:
@@ -478,7 +514,7 @@ class HumanizeTests(SimpleTestCase):
             now + datetime.timedelta(days=2, hours=6),
             now + datetime.timedelta(days=500),
             now.replace(tzinfo=naive()),
-            now.replace(tzinfo=datetime.timezone.utc),
+            now.replace(tzinfo=datetime.UTC),
         ]
         result_list = [
             "test",
@@ -574,8 +610,8 @@ class HumanizeTests(SimpleTestCase):
 
     def test_inflection_for_timedelta(self):
         """
-        Translation of '%d day'/'%d month'/… may differ depending on the context
-        of the string it is inserted in.
+        Translation of '%d day'/'%d month'/… may differ depending on the
+        context of the string it is inserted in.
         """
         test_list = [
             # "%(delta)s ago" translations
