@@ -2,13 +2,19 @@ import datetime
 from unittest import mock
 
 from django.contrib.postgres.indexes import OpClass
+from django.core.checks import Error
 from django.core.exceptions import ValidationError
-from django.db import IntegrityError, NotSupportedError, connection, transaction
+from django.db import IntegrityError, connection, transaction
 from django.db.models import (
+    CASCADE,
+    CharField,
     CheckConstraint,
+    DateField,
     Deferrable,
     F,
+    ForeignKey,
     Func,
+    GeneratedField,
     IntegerField,
     Model,
     Q,
@@ -16,10 +22,9 @@ from django.db.models import (
 )
 from django.db.models.fields.json import KeyTextTransform
 from django.db.models.functions import Cast, Left, Lower
-from django.test import ignore_warnings, skipUnlessDBFeature
+from django.test import skipUnlessDBFeature
 from django.test.utils import isolate_apps
 from django.utils import timezone
-from django.utils.deprecation import RemovedInDjango50Warning
 
 from . import PostgreSQLTestCase
 from .models import HotelReservation, IntegerArrayModel, RangesModel, Room, Scene
@@ -28,6 +33,7 @@ try:
     from django.contrib.postgres.constraints import ExclusionConstraint
     from django.contrib.postgres.fields import (
         DateTimeRangeField,
+        IntegerRangeField,
         RangeBoundary,
         RangeOperators,
     )
@@ -55,7 +61,7 @@ class SchemaTests(PostgreSQLTestCase):
             constraint_name, self.get_constraints(RangesModel._meta.db_table)
         )
         constraint = CheckConstraint(
-            check=Q(ints__contained_by=NumericRange(10, 30)),
+            condition=Q(ints__contained_by=NumericRange(10, 30)),
             name=constraint_name,
         )
         with connection.schema_editor() as editor:
@@ -67,7 +73,7 @@ class SchemaTests(PostgreSQLTestCase):
 
     def test_check_constraint_array_contains(self):
         constraint = CheckConstraint(
-            check=Q(field__contains=[1]),
+            condition=Q(field__contains=[1]),
             name="array_contains",
         )
         msg = f"Constraint “{constraint.name}” is violated."
@@ -77,7 +83,7 @@ class SchemaTests(PostgreSQLTestCase):
 
     def test_check_constraint_array_length(self):
         constraint = CheckConstraint(
-            check=Q(field__len=1),
+            condition=Q(field__len=1),
             name="array_length",
         )
         msg = f"Constraint “{constraint.name}” is violated."
@@ -91,7 +97,7 @@ class SchemaTests(PostgreSQLTestCase):
             constraint_name, self.get_constraints(RangesModel._meta.db_table)
         )
         constraint = CheckConstraint(
-            check=Q(dates__contains=F("dates_inner")),
+            condition=Q(dates__contains=F("dates_inner")),
             name=constraint_name,
         )
         with connection.schema_editor() as editor:
@@ -115,7 +121,7 @@ class SchemaTests(PostgreSQLTestCase):
             constraint_name, self.get_constraints(RangesModel._meta.db_table)
         )
         constraint = CheckConstraint(
-            check=Q(timestamps__contains=F("timestamps_inner")),
+            condition=Q(timestamps__contains=F("timestamps_inner")),
             name=constraint_name,
         )
         with connection.schema_editor() as editor:
@@ -135,7 +141,7 @@ class SchemaTests(PostgreSQLTestCase):
 
     def test_check_constraint_range_contains(self):
         constraint = CheckConstraint(
-            check=Q(ints__contains=(1, 5)),
+            condition=Q(ints__contains=(1, 5)),
             name="ints_contains",
         )
         msg = f"Constraint “{constraint.name}” is violated."
@@ -144,7 +150,7 @@ class SchemaTests(PostgreSQLTestCase):
 
     def test_check_constraint_range_lower_upper(self):
         constraint = CheckConstraint(
-            check=Q(ints__startswith__gte=0) & Q(ints__endswith__lte=99),
+            condition=Q(ints__startswith__gte=0) & Q(ints__endswith__lte=99),
             name="ints_range_lower_upper",
         )
         msg = f"Constraint “{constraint.name}” is violated."
@@ -156,12 +162,12 @@ class SchemaTests(PostgreSQLTestCase):
 
     def test_check_constraint_range_lower_with_nulls(self):
         constraint = CheckConstraint(
-            check=Q(ints__isnull=True) | Q(ints__startswith__gte=0),
+            condition=Q(ints__isnull=True) | Q(ints__startswith__gte=0),
             name="ints_optional_positive_range",
         )
         constraint.validate(RangesModel, RangesModel())
         constraint = CheckConstraint(
-            check=Q(ints__startswith__gte=0),
+            condition=Q(ints__startswith__gte=0),
             name="ints_positive_range",
         )
         constraint.validate(RangesModel, RangesModel())
@@ -262,6 +268,19 @@ class SchemaTests(PostgreSQLTestCase):
         self.assertNotIn(constraint.name, self.get_constraints(Scene._meta.db_table))
         Scene.objects.create(scene="ScEnE 10", setting="Sir Bedemir's Castle")
 
+    def test_opclass_func_validate_constraints(self):
+        constraint_name = "test_opclass_func_validate_constraints"
+        constraint = UniqueConstraint(
+            OpClass(Lower("scene"), name="text_pattern_ops"),
+            name="test_opclass_func_validate_constraints",
+        )
+        Scene.objects.create(scene="First scene")
+        # Non-unique scene.
+        msg = f"Constraint “{constraint_name}” is violated."
+        with self.assertRaisesMessage(ValidationError, msg):
+            constraint.validate(Scene, Scene(scene="first Scene"))
+        constraint.validate(Scene, Scene(scene="second Scene"))
+
 
 class ExclusionConstraintTests(PostgreSQLTestCase):
     def get_constraints(self, table):
@@ -290,7 +309,7 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
 
     def test_invalid_expressions(self):
         msg = "The expressions must be a list of 2-tuples."
-        for expressions in (["foo"], [("foo")], [("foo_1", "foo_2", "foo_3")]):
+        for expressions in (["foo"], [("foo",)], [("foo_1", "foo_2", "foo_3")]):
             with self.subTest(expressions), self.assertRaisesMessage(ValueError, msg):
                 ExclusionConstraint(
                     index_type="GIST",
@@ -301,8 +320,9 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
     def test_empty_expressions(self):
         msg = "At least one expression is required to define an exclusion constraint."
         for empty_expressions in (None, []):
-            with self.subTest(empty_expressions), self.assertRaisesMessage(
-                ValueError, msg
+            with (
+                self.subTest(empty_expressions),
+                self.assertRaisesMessage(ValueError, msg),
             ):
                 ExclusionConstraint(
                     index_type="GIST",
@@ -328,29 +348,56 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
                 include="invalid",
             )
 
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_invalid_opclasses_type(self):
-        msg = "ExclusionConstraint.opclasses must be a list or tuple."
-        with self.assertRaisesMessage(ValueError, msg):
-            ExclusionConstraint(
-                name="exclude_invalid_opclasses",
-                expressions=[(F("datespan"), RangeOperators.OVERLAPS)],
-                opclasses="invalid",
-            )
+    @isolate_apps("postgres_tests")
+    def test_check(self):
+        class Author(Model):
+            name = CharField(max_length=255)
+            alias = CharField(max_length=255)
 
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_opclasses_and_expressions_same_length(self):
-        msg = (
-            "ExclusionConstraint.expressions and "
-            "ExclusionConstraint.opclasses must have the same number of "
-            "elements."
+            class Meta:
+                app_label = "postgres_tests"
+
+        class Book(Model):
+            title = CharField(max_length=255)
+            published_date = DateField()
+            author = ForeignKey(Author, CASCADE)
+
+            class Meta:
+                app_label = "postgres_tests"
+                constraints = [
+                    ExclusionConstraint(
+                        name="exclude_check",
+                        expressions=[
+                            (F("title"), RangeOperators.EQUAL),
+                            (F("published_date__year"), RangeOperators.EQUAL),
+                            ("published_date__month", RangeOperators.EQUAL),
+                            (F("author__name"), RangeOperators.EQUAL),
+                            ("author__alias", RangeOperators.EQUAL),
+                            ("nonexistent", RangeOperators.EQUAL),
+                        ],
+                    )
+                ]
+
+        self.assertCountEqual(
+            Book.check(databases=self.databases),
+            [
+                Error(
+                    "'constraints' refers to the nonexistent field 'nonexistent'.",
+                    obj=Book,
+                    id="models.E012",
+                ),
+                Error(
+                    "'constraints' refers to the joined field 'author__alias'.",
+                    obj=Book,
+                    id="models.E041",
+                ),
+                Error(
+                    "'constraints' refers to the joined field 'author__name'.",
+                    obj=Book,
+                    id="models.E041",
+                ),
+            ],
         )
-        with self.assertRaisesMessage(ValueError, msg):
-            ExclusionConstraint(
-                name="exclude_invalid_expressions_opclasses_length",
-                expressions=[(F("datespan"), RangeOperators.OVERLAPS)],
-                opclasses=["foo", "bar"],
-            )
 
     def test_repr(self):
         constraint = ExclusionConstraint(
@@ -411,6 +458,28 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             "(OpClass(F(datespan), name=range_ops), '-|-')] "
             "name='exclude_overlapping'>",
         )
+        constraint = ExclusionConstraint(
+            name="exclude_overlapping",
+            expressions=[(F("datespan"), RangeOperators.ADJACENT_TO)],
+            violation_error_message="Overlapping must be excluded",
+        )
+        self.assertEqual(
+            repr(constraint),
+            "<ExclusionConstraint: index_type='GIST' expressions=["
+            "(F(datespan), '-|-')] name='exclude_overlapping' "
+            "violation_error_message='Overlapping must be excluded'>",
+        )
+        constraint = ExclusionConstraint(
+            name="exclude_overlapping",
+            expressions=[(F("datespan"), RangeOperators.ADJACENT_TO)],
+            violation_error_code="overlapping_must_be_excluded",
+        )
+        self.assertEqual(
+            repr(constraint),
+            "<ExclusionConstraint: index_type='GIST' expressions=["
+            "(F(datespan), '-|-')] name='exclude_overlapping' "
+            "violation_error_code='overlapping_must_be_excluded'>",
+        )
 
     def test_eq(self):
         constraint_1 = ExclusionConstraint(
@@ -466,27 +535,6 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             ],
             include=["cancelled"],
         )
-        with ignore_warnings(category=RemovedInDjango50Warning):
-            constraint_8 = ExclusionConstraint(
-                name="exclude_overlapping",
-                expressions=[
-                    ("datespan", RangeOperators.OVERLAPS),
-                    ("room", RangeOperators.EQUAL),
-                ],
-                include=["cancelled"],
-                opclasses=["range_ops", "range_ops"],
-            )
-            constraint_9 = ExclusionConstraint(
-                name="exclude_overlapping",
-                expressions=[
-                    ("datespan", RangeOperators.OVERLAPS),
-                    ("room", RangeOperators.EQUAL),
-                ],
-                opclasses=["range_ops", "range_ops"],
-            )
-            self.assertNotEqual(constraint_2, constraint_9)
-            self.assertNotEqual(constraint_7, constraint_8)
-
         constraint_10 = ExclusionConstraint(
             name="exclude_overlapping",
             expressions=[
@@ -505,6 +553,16 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             condition=Q(cancelled=False),
             violation_error_message="other custom error",
         )
+        constraint_12 = ExclusionConstraint(
+            name="exclude_overlapping",
+            expressions=[
+                (F("datespan"), RangeOperators.OVERLAPS),
+                (F("room"), RangeOperators.EQUAL),
+            ],
+            condition=Q(cancelled=False),
+            violation_error_code="custom_code",
+            violation_error_message="other custom error",
+        )
         self.assertEqual(constraint_1, constraint_1)
         self.assertEqual(constraint_1, mock.ANY)
         self.assertNotEqual(constraint_1, constraint_2)
@@ -518,7 +576,9 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
         self.assertNotEqual(constraint_5, constraint_6)
         self.assertNotEqual(constraint_1, object())
         self.assertNotEqual(constraint_10, constraint_11)
+        self.assertNotEqual(constraint_11, constraint_12)
         self.assertEqual(constraint_10, constraint_10)
+        self.assertEqual(constraint_12, constraint_12)
 
     def test_deconstruct(self):
         constraint = ExclusionConstraint(
@@ -636,27 +696,6 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             },
         )
 
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_deconstruct_opclasses(self):
-        constraint = ExclusionConstraint(
-            name="exclude_overlapping",
-            expressions=[("datespan", RangeOperators.OVERLAPS)],
-            opclasses=["range_ops"],
-        )
-        path, args, kwargs = constraint.deconstruct()
-        self.assertEqual(
-            path, "django.contrib.postgres.constraints.ExclusionConstraint"
-        )
-        self.assertEqual(args, ())
-        self.assertEqual(
-            kwargs,
-            {
-                "name": "exclude_overlapping",
-                "expressions": [("datespan", RangeOperators.OVERLAPS)],
-                "opclasses": ["range_ops"],
-            },
-        )
-
     def _test_range_overlaps(self, constraint):
         # Create exclusion constraint.
         self.assertNotIn(
@@ -758,23 +797,17 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             ),
             exclude={"datespan", "start", "end", "room"},
         )
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_range_overlaps_custom_opclasses(self):
-        class TsTzRange(Func):
-            function = "TSTZRANGE"
-            output_field = DateTimeRangeField()
-
-        constraint = ExclusionConstraint(
-            name="exclude_overlapping_reservations_custom",
-            expressions=[
-                (TsTzRange("start", "end", RangeBoundary()), RangeOperators.OVERLAPS),
-                ("room", RangeOperators.EQUAL),
-            ],
-            condition=Q(cancelled=False),
-            opclasses=["range_ops", "gist_int4_ops"],
+        # Constraints with excluded fields in condition are ignored.
+        constraint.validate(
+            HotelReservation,
+            HotelReservation(
+                datespan=(datetimes[1].date(), datetimes[2].date()),
+                start=datetimes[1],
+                end=datetimes[2],
+                room=room102,
+            ),
+            exclude={"cancelled"},
         )
-        self._test_range_overlaps(constraint)
 
     def test_range_overlaps_custom(self):
         class TsTzRange(Func):
@@ -788,7 +821,7 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
                     OpClass(TsTzRange("start", "end", RangeBoundary()), "range_ops"),
                     RangeOperators.OVERLAPS,
                 ),
-                (OpClass("room", "gist_int4_ops"), RangeOperators.EQUAL),
+                (OpClass("room", "gist_int8_ops"), RangeOperators.EQUAL),
             ],
             condition=Q(cancelled=False),
         )
@@ -833,17 +866,64 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
         constraint = ExclusionConstraint(
             name="ints_adjacent",
             expressions=[("ints", RangeOperators.ADJACENT_TO)],
+            violation_error_code="custom_code",
             violation_error_message="Custom error message.",
         )
         range_obj = RangesModel.objects.create(ints=(20, 50))
         constraint.validate(RangesModel, range_obj)
         msg = "Custom error message."
-        with self.assertRaisesMessage(ValidationError, msg):
+        with self.assertRaisesMessage(ValidationError, msg) as cm:
             constraint.validate(RangesModel, RangesModel(ints=(10, 20)))
+        self.assertEqual(cm.exception.code, "custom_code")
         constraint.validate(RangesModel, RangesModel(ints=(10, 19)))
         constraint.validate(RangesModel, RangesModel(ints=(51, 60)))
         constraint.validate(RangesModel, RangesModel(ints=(10, 20)), exclude={"ints"})
 
+    @skipUnlessDBFeature("supports_stored_generated_columns")
+    @isolate_apps("postgres_tests")
+    def test_validate_generated_field_range_adjacent(self):
+        class RangesModelGeneratedField(Model):
+            ints = IntegerRangeField(blank=True, null=True)
+            ints_generated = GeneratedField(
+                expression=F("ints"),
+                output_field=IntegerRangeField(null=True),
+                db_persist=True,
+            )
+
+        with connection.schema_editor() as editor:
+            editor.create_model(RangesModelGeneratedField)
+
+        constraint = ExclusionConstraint(
+            name="ints_adjacent",
+            expressions=[("ints_generated", RangeOperators.ADJACENT_TO)],
+            violation_error_code="custom_code",
+            violation_error_message="Custom error message.",
+        )
+        RangesModelGeneratedField.objects.create(ints=(20, 50))
+
+        range_obj = RangesModelGeneratedField(ints=(3, 20))
+        with self.assertRaisesMessage(ValidationError, "Custom error message."):
+            constraint.validate(RangesModelGeneratedField, range_obj)
+
+        # Excluding referenced or generated field should skip validation.
+        constraint.validate(RangesModelGeneratedField, range_obj, exclude={"ints"})
+        constraint.validate(
+            RangesModelGeneratedField, range_obj, exclude={"ints_generated"}
+        )
+
+    def test_validate_with_custom_code_and_condition(self):
+        constraint = ExclusionConstraint(
+            name="ints_adjacent",
+            expressions=[("ints", RangeOperators.ADJACENT_TO)],
+            violation_error_code="custom_code",
+            condition=Q(ints__lt=(100, 200)),
+        )
+        range_obj = RangesModel.objects.create(ints=(20, 50))
+        constraint.validate(RangesModel, range_obj)
+        with self.assertRaises(ValidationError) as cm:
+            constraint.validate(RangesModel, RangesModel(ints=(10, 20)))
+        self.assertEqual(cm.exception.code, "custom_code")
+
     def test_expressions_with_params(self):
         constraint_name = "scene_left_equal"
         self.assertNotIn(constraint_name, self.get_constraints(Scene._meta.db_table))
@@ -962,7 +1042,6 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
         RangesModel.objects.create(ints=(10, 19))
         RangesModel.objects.create(ints=(51, 60))
 
-    @skipUnlessDBFeature("supports_covering_spgist_indexes")
     def test_range_adjacent_spgist_include(self):
         constraint_name = "ints_adjacent_spgist_include"
         self.assertNotIn(
@@ -999,7 +1078,6 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             editor.add_constraint(RangesModel, constraint)
         self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
 
-    @skipUnlessDBFeature("supports_covering_spgist_indexes")
     def test_range_adjacent_spgist_include_condition(self):
         constraint_name = "ints_adjacent_spgist_include_condition"
         self.assertNotIn(
@@ -1032,7 +1110,6 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             editor.add_constraint(RangesModel, constraint)
         self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
 
-    @skipUnlessDBFeature("supports_covering_spgist_indexes")
     def test_range_adjacent_spgist_include_deferrable(self):
         constraint_name = "ints_adjacent_spgist_include_deferrable"
         self.assertNotIn(
@@ -1049,27 +1126,6 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             editor.add_constraint(RangesModel, constraint)
         self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
 
-    def test_spgist_include_not_supported(self):
-        constraint_name = "ints_adjacent_spgist_include_not_supported"
-        constraint = ExclusionConstraint(
-            name=constraint_name,
-            expressions=[("ints", RangeOperators.ADJACENT_TO)],
-            index_type="spgist",
-            include=["id"],
-        )
-        msg = (
-            "Covering exclusion constraints using an SP-GiST index require "
-            "PostgreSQL 14+."
-        )
-        with connection.schema_editor() as editor:
-            with mock.patch(
-                "django.db.backends.postgresql.features.DatabaseFeatures."
-                "supports_covering_spgist_indexes",
-                False,
-            ):
-                with self.assertRaisesMessage(NotSupportedError, msg):
-                    editor.add_constraint(RangesModel, constraint)
-
     def test_range_adjacent_opclass(self):
         constraint_name = "ints_adjacent_opclass"
         self.assertNotIn(
@@ -1152,7 +1208,6 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             editor.add_constraint(RangesModel, constraint)
         self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
 
-    @skipUnlessDBFeature("supports_covering_spgist_indexes")
     def test_range_adjacent_spgist_opclass_include(self):
         constraint_name = "ints_adjacent_spgist_opclass_include"
         self.assertNotIn(
@@ -1204,136 +1259,11 @@ class ExclusionConstraintTests(PostgreSQLTestCase):
             self.get_constraints(ModelWithExclusionConstraint._meta.db_table),
         )
 
-
-class ExclusionConstraintOpclassesDepracationTests(PostgreSQLTestCase):
-    def get_constraints(self, table):
-        """Get the constraints on the table using a new cursor."""
-        with connection.cursor() as cursor:
-            return connection.introspection.get_constraints(cursor, table)
-
-    def test_warning(self):
-        msg = (
-            "The opclasses argument is deprecated in favor of using "
-            "django.contrib.postgres.indexes.OpClass in "
-            "ExclusionConstraint.expressions."
-        )
-        with self.assertWarnsMessage(RemovedInDjango50Warning, msg):
-            ExclusionConstraint(
-                name="exclude_overlapping",
-                expressions=[(F("datespan"), RangeOperators.ADJACENT_TO)],
-                opclasses=["range_ops"],
-            )
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_repr(self):
+    def test_database_default(self):
         constraint = ExclusionConstraint(
-            name="exclude_overlapping",
-            expressions=[(F("datespan"), RangeOperators.ADJACENT_TO)],
-            opclasses=["range_ops"],
+            name="ints_equal", expressions=[("ints", RangeOperators.EQUAL)]
         )
-        self.assertEqual(
-            repr(constraint),
-            "<ExclusionConstraint: index_type='GIST' expressions=["
-            "(F(datespan), '-|-')] name='exclude_overlapping' "
-            "opclasses=['range_ops']>",
-        )
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_range_adjacent_opclasses(self):
-        constraint_name = "ints_adjacent_opclasses"
-        self.assertNotIn(
-            constraint_name, self.get_constraints(RangesModel._meta.db_table)
-        )
-        constraint = ExclusionConstraint(
-            name=constraint_name,
-            expressions=[("ints", RangeOperators.ADJACENT_TO)],
-            opclasses=["range_ops"],
-        )
-        with connection.schema_editor() as editor:
-            editor.add_constraint(RangesModel, constraint)
-        constraints = self.get_constraints(RangesModel._meta.db_table)
-        self.assertIn(constraint_name, constraints)
-        with editor.connection.cursor() as cursor:
-            cursor.execute(SchemaTests.get_opclass_query, [constraint.name])
-            self.assertEqual(
-                cursor.fetchall(),
-                [("range_ops", constraint.name)],
-            )
-        RangesModel.objects.create(ints=(20, 50))
-        with self.assertRaises(IntegrityError), transaction.atomic():
-            RangesModel.objects.create(ints=(10, 20))
-        RangesModel.objects.create(ints=(10, 19))
-        RangesModel.objects.create(ints=(51, 60))
-        # Drop the constraint.
-        with connection.schema_editor() as editor:
-            editor.remove_constraint(RangesModel, constraint)
-        self.assertNotIn(
-            constraint_name, self.get_constraints(RangesModel._meta.db_table)
-        )
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_range_adjacent_opclasses_condition(self):
-        constraint_name = "ints_adjacent_opclasses_condition"
-        self.assertNotIn(
-            constraint_name, self.get_constraints(RangesModel._meta.db_table)
-        )
-        constraint = ExclusionConstraint(
-            name=constraint_name,
-            expressions=[("ints", RangeOperators.ADJACENT_TO)],
-            opclasses=["range_ops"],
-            condition=Q(id__gte=100),
-        )
-        with connection.schema_editor() as editor:
-            editor.add_constraint(RangesModel, constraint)
-        self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_range_adjacent_opclasses_deferrable(self):
-        constraint_name = "ints_adjacent_opclasses_deferrable"
-        self.assertNotIn(
-            constraint_name, self.get_constraints(RangesModel._meta.db_table)
-        )
-        constraint = ExclusionConstraint(
-            name=constraint_name,
-            expressions=[("ints", RangeOperators.ADJACENT_TO)],
-            opclasses=["range_ops"],
-            deferrable=Deferrable.DEFERRED,
-        )
-        with connection.schema_editor() as editor:
-            editor.add_constraint(RangesModel, constraint)
-        self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_range_adjacent_gist_opclasses_include(self):
-        constraint_name = "ints_adjacent_gist_opclasses_include"
-        self.assertNotIn(
-            constraint_name, self.get_constraints(RangesModel._meta.db_table)
-        )
-        constraint = ExclusionConstraint(
-            name=constraint_name,
-            expressions=[("ints", RangeOperators.ADJACENT_TO)],
-            index_type="gist",
-            opclasses=["range_ops"],
-            include=["decimals"],
-        )
-        with connection.schema_editor() as editor:
-            editor.add_constraint(RangesModel, constraint)
-        self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    @skipUnlessDBFeature("supports_covering_spgist_indexes")
-    def test_range_adjacent_spgist_opclasses_include(self):
-        constraint_name = "ints_adjacent_spgist_opclasses_include"
-        self.assertNotIn(
-            constraint_name, self.get_constraints(RangesModel._meta.db_table)
-        )
-        constraint = ExclusionConstraint(
-            name=constraint_name,
-            expressions=[("ints", RangeOperators.ADJACENT_TO)],
-            index_type="spgist",
-            opclasses=["range_ops"],
-            include=["decimals"],
-        )
-        with connection.schema_editor() as editor:
-            editor.add_constraint(RangesModel, constraint)
-        self.assertIn(constraint_name, self.get_constraints(RangesModel._meta.db_table))
+        RangesModel.objects.create()
+        msg = "Constraint “ints_equal” is violated."
+        with self.assertRaisesMessage(ValidationError, msg):
+            constraint.validate(RangesModel, RangesModel())
