@@ -4,25 +4,24 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from django.db import NotSupportedError, connection
 from django.db.models import F, Prefetch, QuerySet, prefetch_related_objects
-from django.db.models.fields.related import ForwardManyToOneDescriptor
-from django.db.models.query import get_prefetcher, prefetch_one_level
+from django.db.models.query import get_prefetcher
 from django.db.models.sql import Query
 from django.test import (
     TestCase,
-    ignore_warnings,
     override_settings,
     skipIfDBFeature,
     skipUnlessDBFeature,
 )
 from django.test.utils import CaptureQueriesContext
-from django.utils.deprecation import RemovedInDjango60Warning
 
 from .models import (
     Article,
+    ArticleCustomUUID,
     Author,
     Author2,
     AuthorAddress,
     AuthorWithAge,
+    AuthorWithAgeChild,
     Bio,
     Book,
     Bookmark,
@@ -487,9 +486,10 @@ class CustomPrefetchTests(TestCase):
     @classmethod
     def traverse_qs(cls, obj_iter, path):
         """
-        Helper method that returns a list containing a list of the objects in the
-        obj_iter. Then for each object in the obj_iter, the path will be
-        recursively travelled and the found objects are added to the return value.
+        Helper method that returns a list containing a list of the objects in
+        the obj_iter. Then for each object in the obj_iter, the path will be
+        recursively travelled and the found objects are added to the return
+        value.
         """
         ret_val = []
 
@@ -1065,7 +1065,8 @@ class CustomPrefetchTests(TestCase):
             Prefetch("houses", House.objects.values("pk"))
         with self.assertRaisesMessage(ValueError, msg):
             Prefetch("houses", House.objects.values_list("pk"))
-        # That error doesn't affect managers with custom ModelIterable subclasses
+        # That error doesn't affect managers with custom ModelIterable
+        # subclasses
         self.assertIs(
             Teacher.objects_custom.all()._iterable_class, ModelIterableSubclass
         )
@@ -1135,8 +1136,8 @@ class DefaultManagerTests(TestCase):
     def test_m2m_then_m2m(self):
         with self.assertNumQueries(3):
             # When we prefetch the teachers, and force the query, we don't want
-            # the default manager on teachers to immediately get all the related
-            # qualifications, since this will do one query per teacher.
+            # the default manager on teachers to immediately get all the
+            # related qualifications, since this will do one query per teacher.
             qs = Department.objects.prefetch_related("teachers")
             depts = "".join(
                 "%s department: %s\n"
@@ -1195,6 +1196,14 @@ class GenericRelationTests(TestCase):
         qs = Comment.objects.prefetch_related("content_object_uuid")
         self.assertEqual([c.content_object_uuid for c in qs], [article])
 
+    def test_prefetch_GFK_uses_prepped_primary_key(self):
+        article = ArticleCustomUUID.objects.create(name="Blanche")
+        Comment.objects.create(comment="Enchantment", content_object_uuid=article)
+        obj = Comment.objects.prefetch_related("content_object_uuid").get(
+            comment="Enchantment"
+        )
+        self.assertEqual(obj.content_object_uuid, article)
+
     def test_prefetch_GFK_fk_pk(self):
         book = Book.objects.create(title="Poems")
         book_with_year = BookWithYear.objects.create(book=book, published_year=2019)
@@ -1389,9 +1398,9 @@ class MultiTableInheritanceTest(TestCase):
                 for a in Author.objects.prefetch_related("authorwithage")
             ]
 
-        # Regression for #18090: the prefetching query must include an IN clause.
-        # Note that on Oracle the table name is upper case in the generated SQL,
-        # thus the .lower() call.
+        # Regression for #18090: the prefetching query must include an IN
+        # clause. Note that on Oracle the table name is upper case in the
+        # generated SQL, thus the .lower() call.
         self.assertIn("authorwithage", connection.queries[-1]["sql"].lower())
         self.assertIn(" IN ", connection.queries[-1]["sql"])
 
@@ -1485,8 +1494,9 @@ class LookupOrderingTest(TestCase):
 
     def test_order(self):
         with self.assertNumQueries(4):
-            # The following two queries must be done in the same order as written,
-            # otherwise 'primary_house' will cause non-prefetched lookups
+            # The following two queries must be done in the same order as
+            # written, otherwise 'primary_house' will cause non-prefetched
+            # lookups
             qs = Person.objects.prefetch_related(
                 "houses__rooms", "primary_house__occupants"
             )
@@ -1502,7 +1512,8 @@ class NullableTest(TestCase):
 
     def test_traverse_nullable(self):
         # Because we use select_related() for 'boss', it doesn't need to be
-        # prefetched, but we can still traverse it although it contains some nulls
+        # prefetched, but we can still traverse it although it contains some
+        # nulls
         with self.assertNumQueries(2):
             qs = Employee.objects.select_related("boss").prefetch_related("boss__serfs")
             co_serfs = [
@@ -1809,8 +1820,8 @@ class DirectPrefetchedObjectCacheReuseTests(TestCase):
 
     def test_detect_is_fetched(self):
         """
-        Nested prefetch_related() shouldn't trigger duplicate queries for the same
-        lookup.
+        Nested prefetch_related() shouldn't trigger duplicate queries for the
+        same lookup.
         """
         with self.assertNumQueries(3):
             books = Book.objects.filter(title__in=["book1", "book2"]).prefetch_related(
@@ -2082,57 +2093,182 @@ class PrefetchLimitTests(TestDataMixin, TestCase):
                 self.assertIn(book.authors_sliced[0], list(book.authors.all()))
 
 
-class DeprecationTests(TestCase):
-    def test_get_current_queryset_warning(self):
-        msg = (
-            "Prefetch.get_current_queryset() is deprecated. Use "
-            "get_current_querysets() instead."
+class PrefetchRelatedMTICacheTests(TestCase):
+    @classmethod
+    def setUpTestData(cls):
+        cls.book = Book.objects.create(title="Book1")
+        cls.related1 = Author.objects.create(name="related1", first_book=cls.book)
+        cls.related2 = Author.objects.create(name="related2", first_book=cls.book)
+        cls.related3 = Author.objects.create(name="related3", first_book=cls.book)
+        cls.related4 = Author.objects.create(name="related4", first_book=cls.book)
+
+        cls.child = AuthorWithAgeChild.objects.create(
+            name="child",
+            age=31,
+            first_book=cls.book,
         )
-        authors = Author.objects.all()
-        with self.assertWarnsMessage(RemovedInDjango60Warning, msg) as ctx:
-            self.assertEqual(
-                Prefetch("authors", authors).get_current_queryset(1),
-                authors,
+        cls.m2m_child = AuthorWithAgeChild.objects.create(
+            name="m2m_child",
+            age=31,
+            first_book=cls.book,
+        )
+        cls.m2m_child.favorite_authors.set([cls.related1, cls.related2, cls.related3])
+
+    def test_parent_fk_available_in_child(self):
+        qs = (
+            Author.objects.select_related("authorwithage")
+            .prefetch_related("first_book")
+            .filter(pk=self.child.pk)
+        )
+        with self.assertNumQueries(2):
+            results = list(qs)
+            self.assertEqual(len(results), 1)
+            self.assertEqual(results[0].authorwithage.first_book, self.book)
+
+    def test_grandparent_fk_available_in_child(self):
+        qs = (
+            Author.objects.select_related(
+                "authorwithage", "authorwithage__authorwithagechild"
             )
-        self.assertEqual(ctx.filename, __file__)
-        with self.assertWarnsMessage(RemovedInDjango60Warning, msg) as ctx:
-            self.assertIsNone(Prefetch("authors").get_current_queryset(1))
-        self.assertEqual(ctx.filename, __file__)
-
-    @ignore_warnings(category=RemovedInDjango60Warning)
-    def test_prefetch_one_level_fallback(self):
-        class NoGetPrefetchQuerySetsDescriptor(ForwardManyToOneDescriptor):
-            def get_prefetch_queryset(self, instances, queryset=None):
-                if queryset is None:
-                    return super().get_prefetch_querysets(instances)
-                return super().get_prefetch_querysets(instances, [queryset])
-
-            def __getattribute__(self, name):
-                if name == "get_prefetch_querysets":
-                    raise AttributeError
-                return super().__getattribute__(name)
-
-        house = House.objects.create()
-        room = Room.objects.create(house=house)
-        house.main_room = room
-        house.save()
-
-        # prefetch_one_level() fallbacks to get_prefetch_queryset().
-        prefetcher = NoGetPrefetchQuerySetsDescriptor(Room._meta.get_field("house"))
-        obj_list, additional_lookups = prefetch_one_level(
-            [room],
-            prefetcher,
-            Prefetch("house", House.objects.all()),
-            0,
+            .prefetch_related("first_book")
+            .filter(pk=self.child.pk)
         )
-        self.assertEqual(obj_list, [house])
-        self.assertEqual(additional_lookups, [])
+        with self.assertNumQueries(2):
+            results = list(qs)
+            self.assertEqual(len(results), 1)
+            self.assertEqual(
+                results[0].authorwithage.authorwithagechild.first_book, self.book
+            )
 
-        obj_list, additional_lookups = prefetch_one_level(
-            [room],
-            prefetcher,
-            Prefetch("house"),
-            0,
+    def test_parent_m2m_available_in_child(self):
+        qs = (
+            Author.objects.select_related("authorwithage")
+            .prefetch_related("favorite_authors")
+            .filter(pk=self.m2m_child.pk)
+        )
+        with self.assertNumQueries(2):
+            results = list(qs)
+            self.assertEqual(len(results), 1)
+            self.assertQuerySetEqual(
+                results[0].authorwithage.favorite_authors.all(),
+                [self.related1, self.related2, self.related3],
+            )
+
+    def test_grandparent_m2m_available_in_child(self):
+        qs = (
+            Author.objects.select_related(
+                "authorwithage", "authorwithage__authorwithagechild"
+            )
+            .prefetch_related("favorite_authors")
+            .filter(pk=self.m2m_child.pk)
+        )
+        with self.assertNumQueries(2):
+            results = list(qs)
+            self.assertEqual(len(results), 1)
+            self.assertQuerySetEqual(
+                set(results[0].authorwithage.authorwithagechild.favorite_authors.all()),
+                {self.related1, self.related2, self.related3},
+            )
+
+    def test_add_clears_prefetched_objects_in_parent(self):
+        gp = (
+            Author.objects.select_related("authorwithage")
+            .prefetch_related("favorite_authors")
+            .get(pk=self.m2m_child.pk)
+        )
+        self.assertCountEqual(
+            gp.favorite_authors.all(),
+            {self.related1, self.related2, self.related3},
+        )
+        self.assertCountEqual(
+            gp.authorwithage.favorite_authors.all(),
+            {self.related1, self.related2, self.related3},
+        )
+        gp.authorwithage.favorite_authors.add(self.related4)
+        self.assertCountEqual(
+            gp.favorite_authors.all(),
+            {self.related1, self.related2, self.related3, self.related4},
+        )
+        self.assertCountEqual(
+            gp.authorwithage.favorite_authors.all(),
+            {self.related1, self.related2, self.related3, self.related4},
+        )
+
+    def test_add_clears_prefetched_objects_in_grandparent(self):
+        gp = (
+            Author.objects.select_related(
+                "authorwithage", "authorwithage__authorwithagechild"
+            )
+            .prefetch_related("favorite_authors")
+            .get(pk=self.m2m_child.pk)
+        )
+        self.assertCountEqual(
+            gp.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        self.assertCountEqual(
+            gp.authorwithage.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        self.assertCountEqual(
+            gp.authorwithage.authorwithagechild.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        gp.authorwithage.authorwithagechild.favorite_authors.add(self.related4)
+        self.assertCountEqual(
+            gp.favorite_authors.all(),
+            [self.related1, self.related2, self.related3, self.related4],
+        )
+        self.assertCountEqual(
+            gp.authorwithage.favorite_authors.all(),
+            [self.related1, self.related2, self.related3, self.related4],
+        )
+        self.assertCountEqual(
+            gp.authorwithage.authorwithagechild.favorite_authors.all(),
+            [self.related1, self.related2, self.related3, self.related4],
+        )
+
+    def test_remove_clears_prefetched_objects_in_parent(self):
+        gp = (
+            Author.objects.select_related("authorwithage")
+            .prefetch_related("favorite_authors")
+            .get(pk=self.m2m_child.pk)
+        )
+        self.assertCountEqual(
+            gp.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        self.assertCountEqual(
+            gp.authorwithage.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        gp.authorwithage.favorite_authors.clear()
+        self.assertSequenceEqual(gp.favorite_authors.all(), [])
+        self.assertSequenceEqual(gp.authorwithage.favorite_authors.all(), [])
+
+    def test_remove_clears_prefetched_objects_in_grandparent(self):
+        gp = (
+            Author.objects.select_related(
+                "authorwithage", "authorwithage__authorwithagechild"
+            )
+            .prefetch_related("favorite_authors")
+            .get(pk=self.m2m_child.pk)
+        )
+        self.assertCountEqual(
+            gp.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        self.assertCountEqual(
+            gp.authorwithage.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        self.assertCountEqual(
+            gp.authorwithage.authorwithagechild.favorite_authors.all(),
+            [self.related1, self.related2, self.related3],
+        )
+        gp.authorwithage.favorite_authors.clear()
+        self.assertSequenceEqual(gp.favorite_authors.all(), [])
+        self.assertSequenceEqual(gp.authorwithage.favorite_authors.all(), [])
+        self.assertSequenceEqual(
+            gp.authorwithage.authorwithagechild.favorite_authors.all(), []
         )
-        self.assertEqual(obj_list, [house])
-        self.assertEqual(additional_lookups, [])
