@@ -514,9 +514,11 @@ class SimpleBlockTagTests(TagTestCase):
 
     def test_simple_block_tag_with_context_missing_content(self):
         # The 'content' parameter must be present when takes_context is True
-        msg = "'simple_block_tag_with_context_without_content' is decorated with "
-        "takes_context=True so it must have a first argument of 'context' and a "
-        "second argument of 'content'"
+        msg = (
+            "'simple_block_tag_with_context_without_content' is decorated with "
+            "takes_context=True so it must have a first argument of 'context' and a "
+            "second argument of 'content'"
+        )
         with self.assertRaisesMessage(TemplateSyntaxError, msg):
             self.engine.from_string(
                 "{% load custom %}{% simple_block_tag_with_context_without_content %}"
