@@ -13,6 +13,7 @@ from django.template.base import (
     Token,
     TokenType,
     Variable,
+    VariableDoesNotExist,
 )
 from django.template.defaultfilters import register as filter_library
 from django.test import SimpleTestCase
@@ -75,6 +76,27 @@ class ParserTests(SimpleTestCase):
         with self.assertRaisesMessage(TemplateSyntaxError, msg):
             FilterExpression("article._hidden|upper", p)
 
+    def test_cannot_parse_characters(self):
+        p = Parser("", builtins=[filter_library])
+        for filter_expression, characters in [
+            ('<>|default:"Default"|upper', '|<>||default:"Default"|upper'),
+            ("test|<>|upper", "test||<>||upper"),
+        ]:
+            with self.subTest(filter_expression=filter_expression):
+                with self.assertRaisesMessage(
+                    TemplateSyntaxError,
+                    f"Could not parse some characters: {characters}",
+                ):
+                    FilterExpression(filter_expression, p)
+
+    def test_cannot_find_variable(self):
+        p = Parser("", builtins=[filter_library])
+        with self.assertRaisesMessage(
+            TemplateSyntaxError,
+            'Could not find variable at start of |default:"Default"',
+        ):
+            FilterExpression('|default:"Default"', p)
+
     def test_variable_parsing(self):
         c = {"article": {"section": "News"}}
         self.assertEqual(Variable("article.section").resolve(c), "News")
@@ -107,6 +129,16 @@ class ParserTests(SimpleTestCase):
         ):
             Variable({})
 
+        # Variables should raise when invalid characters in name.
+        for c in ["+", "-"]:
+            with self.subTest(invalid_character=c):
+                variable_name = f"variable{c}name"
+                with self.assertRaisesMessage(
+                    TemplateSyntaxError,
+                    f"Invalid character ('{c}') in variable name: '{variable_name}'",
+                ):
+                    Variable(variable_name)
+
     def test_filter_args_count(self):
         parser = Parser("")
         register = Library()
@@ -152,6 +184,66 @@ class ParserTests(SimpleTestCase):
         ):
             FilterExpression(expr, parser)
 
+    def test_filter_numeric_argument_parsing(self):
+        p = Parser("", builtins=[filter_library])
+
+        # Values that resolve to a numeric literal.
+        cases = {
+            "5": 5,
+            "-5": -5,
+            "5.2": 5.2,
+            ".4": 0.4,
+            "5.2e3": 5200.0,  # 5.2 × 10³ = 5200.0.
+            "5.2E3": 5200.0,  # Case-insensitive.
+            "5.2e-3": 0.0052,  # Negative exponent.
+            "-1.5E4": -15000.0,
+            "+3.0e2": 300.0,
+            ".5e2": 50.0,  # 0.5 × 10² = 50.0
+        }
+        for num, expected in cases.items():
+            with self.subTest(num=num):
+                self.assertEqual(FilterExpression(num, p).resolve({}), expected)
+                self.assertEqual(
+                    FilterExpression(f"0|default:{num}", p).resolve({}), expected
+                )
+
+        # Values that are interpreted as names of variables that do not exist.
+        invalid_numbers = [
+            "abc123",
+            "123abc",
+            "foo",
+            "error",
+            "1e",
+            "e400",
+            "1e.2",
+            "1e2.",
+            "1e2.0",
+            "1e2a",
+            "1e2e3",
+        ]
+
+        for num in invalid_numbers:
+            with self.subTest(num=num):
+                self.assertIsNone(
+                    FilterExpression(num, p).resolve({}, ignore_failures=True)
+                )
+                with self.assertRaises(VariableDoesNotExist):
+                    FilterExpression(f"0|default:{num}", p).resolve({})
+
+        # Values that are interpreted as an invalid variable name.
+        invalid_numbers_and_var_names = [
+            "1e-",
+            "1e-a",
+            "1+1",
+            "1-1",
+        ]
+        for num in invalid_numbers_and_var_names:
+            with self.subTest(num=num):
+                with self.assertRaises(TemplateSyntaxError):
+                    FilterExpression(num, p).resolve({})
+                with self.assertRaises(TemplateSyntaxError):
+                    FilterExpression(f"0|default:{num}", p).resolve({})
+
     @unittest.skipUnless(PY314, "Deferred annotations are Python 3.14+ only")
     def test_filter_deferred_annotation(self):
         register = Library()
