@@ -19,6 +19,7 @@ testing against the contexts and templates produced by a view,
 rather than the HTML rendered to the end-user.
 
 """
+
 import copy
 import itertools
 import tempfile
@@ -330,7 +331,9 @@ class ClientTest(TestCase):
         self.assertEqual(response.request["PATH_INFO"], "/accounts/login/")
 
     def test_follow_relative_redirect_no_trailing_slash(self):
-        "A URL with a relative redirect with no trailing slash can be followed."
+        """
+        A URL with a relative redirect with no trailing slash can be followed.
+        """
         response = self.client.get("/accounts/no_trailing_slash", follow=True)
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.request["PATH_INFO"], "/accounts/login/")
@@ -680,7 +683,9 @@ class ClientTest(TestCase):
         ]
     )
     def test_view_with_inactive_force_login(self):
-        "Request a page that is protected with @login, but use an inactive login"
+        """
+        Request a page that is protected with @login, but use an inactive login
+        """
 
         # Get the page without logging in. Should result in 302.
         response = self.client.get("/login_protected_view/")
@@ -856,6 +861,13 @@ class ClientTest(TestCase):
             response, "https://www.djangoproject.com/", fetch_redirect_response=False
         )
 
+    @override_settings(ALLOWED_HOSTS=["hostname1", "hostname2"])
+    def test_redirect_with_http_host(self):
+        response = self.client.get(
+            "/redirect_to_different_hostname/", follow=True, HTTP_HOST="hostname1"
+        )
+        self.assertEqual(response.content, b"hostname2")
+
     def test_external_redirect_without_trailing_slash(self):
         """
         Client._handle_redirects() with an empty path.
@@ -995,6 +1007,36 @@ class ClientTest(TestCase):
             )
         self.assertEqual(response.content, b"named_temp_file")
 
+    def test_query_params(self):
+        tests = (
+            "get",
+            "post",
+            "put",
+            "patch",
+            "delete",
+            "head",
+            "options",
+            "trace",
+        )
+        for method in tests:
+            with self.subTest(method=method):
+                client_method = getattr(self.client, method)
+                response = client_method("/get_view/", query_params={"example": "data"})
+                self.assertEqual(response.wsgi_request.GET["example"], "data")
+
+    def test_cannot_use_data_and_query_params_together(self):
+        tests = ["get", "head"]
+        msg = "query_params and data arguments are mutually exclusive."
+        for method in tests:
+            with self.subTest(method=method):
+                client_method = getattr(self.client, method)
+                with self.assertRaisesMessage(ValueError, msg):
+                    client_method(
+                        "/get_view/",
+                        data={"example": "data"},
+                        query_params={"q": "terms"},
+                    )
+
 
 @override_settings(
     MIDDLEWARE=["django.middleware.csrf.CsrfViewMiddleware"],
@@ -1120,6 +1162,23 @@ class RequestFactoryTest(SimpleTestCase):
             self.assertEqual(request.headers["x-another-header"], "some other value")
             self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
 
+    def test_request_factory_query_params(self):
+        tests = (
+            "get",
+            "post",
+            "put",
+            "patch",
+            "delete",
+            "head",
+            "options",
+            "trace",
+        )
+        for method in tests:
+            with self.subTest(method=method):
+                factory = getattr(self.request_factory, method)
+                request = factory("/somewhere", query_params={"example": "data"})
+                self.assertEqual(request.GET["example"], "data")
+
 
 @override_settings(ROOT_URLCONF="test_client.urls")
 class AsyncClientTest(TestCase):
@@ -1135,8 +1194,11 @@ class AsyncClientTest(TestCase):
         response = await self.async_client.get("/middleware_urlconf_view/")
         self.assertEqual(response.resolver_match.url_name, "middleware_urlconf_view")
 
-    async def test_follow_parameter_not_implemented(self):
-        msg = "AsyncClient request methods do not accept the follow parameter."
+    async def test_redirect(self):
+        response = await self.async_client.get("/redirect_view/")
+        self.assertEqual(response.status_code, 302)
+
+    async def test_follow_redirect(self):
         tests = (
             "get",
             "post",
@@ -1150,8 +1212,16 @@ class AsyncClientTest(TestCase):
         for method_name in tests:
             with self.subTest(method=method_name):
                 method = getattr(self.async_client, method_name)
-                with self.assertRaisesMessage(NotImplementedError, msg):
-                    await method("/redirect_view/", follow=True)
+                response = await method("/redirect_view/", follow=True)
+                self.assertEqual(response.status_code, 200)
+                self.assertEqual(response.resolver_match.url_name, "get_view")
+
+    async def test_follow_double_redirect(self):
+        response = await self.async_client.get("/double_redirect_view/", follow=True)
+        self.assertRedirects(
+            response, "/get_view/", status_code=302, target_status_code=200
+        )
+        self.assertEqual(len(response.redirect_chain), 2)
 
     async def test_get_data(self):
         response = await self.async_client.get("/get_view/", {"var": "val"})
@@ -1165,6 +1235,25 @@ class AsyncClientTest(TestCase):
         response = await self.async_client.get("/post_view/")
         self.assertContains(response, "Viewing GET page.")
 
+    async def test_query_params(self):
+        tests = (
+            "get",
+            "post",
+            "put",
+            "patch",
+            "delete",
+            "head",
+            "options",
+            "trace",
+        )
+        for method in tests:
+            with self.subTest(method=method):
+                client_method = getattr(self.async_client, method)
+                response = await client_method(
+                    "/async_get_view/", query_params={"example": "data"}
+                )
+                self.assertEqual(response.asgi_request.GET["example"], "data")
+
 
 @override_settings(ROOT_URLCONF="test_client.urls")
 class AsyncRequestFactoryTest(SimpleTestCase):
@@ -1242,7 +1331,50 @@ class AsyncRequestFactoryTest(SimpleTestCase):
         self.assertEqual(request.headers["x-another-header"], "some other value")
         self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
 
+    def test_async_request_factory_default_headers(self):
+        request_factory_with_headers = AsyncRequestFactory(
+            **{
+                "Authorization": "Bearer faketoken",
+                "X-Another-Header": "some other value",
+            }
+        )
+        request = request_factory_with_headers.get("/somewhere/")
+        self.assertEqual(request.headers["authorization"], "Bearer faketoken")
+        self.assertIn("HTTP_AUTHORIZATION", request.META)
+        self.assertEqual(request.headers["x-another-header"], "some other value")
+        self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
+
     def test_request_factory_query_string(self):
         request = self.request_factory.get("/somewhere/", {"example": "data"})
         self.assertNotIn("Query-String", request.headers)
         self.assertEqual(request.GET["example"], "data")
+
+    def test_request_factory_query_params(self):
+        tests = (
+            "get",
+            "post",
+            "put",
+            "patch",
+            "delete",
+            "head",
+            "options",
+            "trace",
+        )
+        for method in tests:
+            with self.subTest(method=method):
+                factory = getattr(self.request_factory, method)
+                request = factory("/somewhere", query_params={"example": "data"})
+                self.assertEqual(request.GET["example"], "data")
+
+    def test_cannot_use_data_and_query_params_together(self):
+        tests = ["get", "head"]
+        msg = "query_params and data arguments are mutually exclusive."
+        for method in tests:
+            with self.subTest(method=method):
+                factory = getattr(self.request_factory, method)
+                with self.assertRaisesMessage(ValueError, msg):
+                    factory(
+                        "/somewhere",
+                        data={"example": "data"},
+                        query_params={"q": "terms"},
+                    )
