@@ -1,6 +1,8 @@
+from django.core.exceptions import ObjectNotUpdated
+from django.db import DatabaseError, connection, transaction
+from django.db.models import F
 from django.db.models.signals import post_save, pre_save
 from django.test import TestCase
-from django.utils.deprecation import RemovedInDjango60Warning
 
 from .models import Account, Employee, Person, Profile, ProxyEmployee
 
@@ -257,31 +259,6 @@ class UpdateOnlyFieldsTests(TestCase):
         pre_save.disconnect(pre_save_receiver)
         post_save.disconnect(post_save_receiver)
 
-    def test_empty_update_fields_positional_save(self):
-        s = Person.objects.create(name="Sara", gender="F")
-
-        msg = "Passing positional arguments to save() is deprecated"
-        with (
-            self.assertWarnsMessage(RemovedInDjango60Warning, msg) as ctx,
-            self.assertNumQueries(0),
-        ):
-            s.save(False, False, None, [])
-        self.assertEqual(ctx.filename, __file__)
-
-    async def test_empty_update_fields_positional_asave(self):
-        s = await Person.objects.acreate(name="Sara", gender="F")
-        # Workaround for a lack of async assertNumQueries.
-        s.name = "Other"
-
-        msg = "Passing positional arguments to asave() is deprecated"
-        with self.assertWarnsMessage(RemovedInDjango60Warning, msg) as ctx:
-            await s.asave(False, False, None, [])
-        self.assertEqual(ctx.filename, __file__)
-
-        # No save occurred for an empty update_fields.
-        await s.arefresh_from_db()
-        self.assertEqual(s.name, "Sara")
-
     def test_num_queries_inheritance(self):
         s = Employee.objects.create(name="Sara", gender="F")
         s.employee_num = 1
@@ -318,3 +295,30 @@ class UpdateOnlyFieldsTests(TestCase):
         employee_boss = Employee.objects.create(name="Boss", gender="F")
         with self.assertRaisesMessage(ValueError, self.msg % "id"):
             employee_boss.save(update_fields=["id"])
+
+    def test_update_fields_not_updated(self):
+        obj = Person.objects.create(name="Sara", gender="F")
+        Person.objects.filter(pk=obj.pk).delete()
+        msg = "Save with update_fields did not affect any rows."
+        # Make sure backward compatibility with DatabaseError is preserved.
+        exceptions = [DatabaseError, ObjectNotUpdated, Person.NotUpdated]
+        for exception in exceptions:
+            with (
+                self.subTest(exception),
+                self.assertRaisesMessage(DatabaseError, msg),
+                transaction.atomic(),
+            ):
+                obj.save(update_fields=["name"])
+
+    def test_update_fields_expression(self):
+        obj = Person.objects.create(name="Valerie", gender="F", pid=42)
+        updated_pid = F("pid") + 1
+        obj.pid = updated_pid
+        obj.save(update_fields={"gender"})
+        self.assertIs(obj.pid, updated_pid)
+        obj.save(update_fields={"pid"})
+        expected_num_queries = (
+            0 if connection.features.can_return_rows_from_update else 1
+        )
+        with self.assertNumQueries(expected_num_queries):
+            self.assertEqual(obj.pid, 43)
