@@ -7,7 +7,7 @@ import tempfile
 import threading
 from io import StringIO
 from pathlib import Path
-from unittest import mock, skipIf, skipUnless
+from unittest import mock, skipIf
 
 from asgiref.sync import async_to_sync, iscoroutinefunction
 
@@ -24,7 +24,6 @@ from django.urls.converters import IntConverter
 from django.utils.functional import SimpleLazyObject
 from django.utils.regex_helper import _lazy_re_compile
 from django.utils.safestring import mark_safe
-from django.utils.version import PY311
 from django.views.debug import (
     CallableSettingWrapper,
     ExceptionCycleWarning,
@@ -262,6 +261,22 @@ class DebugViewTests(SimpleTestCase):
             status_code=500,
         )
 
+    def test_technical_500_content_type_negotiation(self):
+        for accepts, content_type in [
+            ("text/plain", "text/plain; charset=utf-8"),
+            ("text/html", "text/html"),
+            ("text/html,text/plain;q=0.9", "text/html"),
+            ("text/plain,text/html;q=0.9", "text/plain; charset=utf-8"),
+            ("text/*", "text/html"),
+        ]:
+            with self.subTest(accepts=accepts):
+                with self.assertLogs("django.request", "ERROR"):
+                    response = self.client.get(
+                        "/raises500/", headers={"accept": accepts}
+                    )
+                self.assertEqual(response.status_code, 500)
+                self.assertEqual(response["Content-Type"], content_type)
+
     def test_classbased_technical_500(self):
         with self.assertLogs("django.request", "ERROR"):
             response = self.client.get("/classbased500/")
@@ -381,7 +396,8 @@ class DebugViewTests(SimpleTestCase):
 
     def test_no_template_source_loaders(self):
         """
-        Make sure if you don't specify a template, the debug view doesn't blow up.
+        Make sure if you don't specify a template, the debug view doesn't blow
+        up.
         """
         with self.assertLogs("django.request", "ERROR"):
             with self.assertRaises(TemplateDoesNotExist):
@@ -478,7 +494,8 @@ class DebugViewQueriesAllowedTests(SimpleTestCase):
     def test_handle_db_exception(self):
         """
         Ensure the debug view works when a database exception is raised by
-        performing an invalid query and passing the exception to the debug view.
+        performing an invalid query and passing the exception to the debug
+        view.
         """
         with connection.cursor() as cursor:
             try:
@@ -599,7 +616,9 @@ class ExceptionReporterTests(SimpleTestCase):
         )
 
     def test_eol_support(self):
-        """The ExceptionReporter supports Unix, Windows and Macintosh EOL markers"""
+        """
+        The ExceptionReporter supports Unix, Windows and Macintosh EOL markers
+        """
         LINES = ["print %d" % i for i in range(1, 6)]
         reporter = ExceptionReporter(None, None, None, None)
 
@@ -695,7 +714,6 @@ class ExceptionReporterTests(SimpleTestCase):
             text,
         )
 
-    @skipUnless(PY311, "Exception notes were added in Python 3.11.")
     def test_exception_with_notes(self):
         request = self.rf.get("/test_view/")
         try:
@@ -806,7 +824,6 @@ class ExceptionReporterTests(SimpleTestCase):
         or os.environ.get("PYTHONNODEBUGRANGES", False),
         "Fine-grained error locations are disabled.",
     )
-    @skipUnless(PY311, "Fine-grained error locations were added in Python 3.11.")
     def test_highlight_error_position(self):
         request = self.rf.get("/test_view/")
         try:
@@ -1027,7 +1044,10 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn("<p>Request data not supplied</p>", html)
 
     def test_non_utf8_values_handling(self):
-        "Non-UTF-8 exceptions/values should not make the output generation choke."
+        """
+        Non-UTF-8 exceptions/values should not make the output generation
+        choke.
+        """
         try:
 
             class NonUtf8Output(Exception):
@@ -1433,7 +1453,8 @@ class ExceptionReportTestMixin:
         self, view, check_for_vars=True, check_for_POST_params=True
     ):
         """
-        Asserts that no variables or POST parameters are displayed in the response.
+        Asserts that no variables or POST parameters are displayed in the
+        response.
         """
         request = self.rf.post("/some_url/", self.breakfast_data)
         response = view(request)
@@ -1452,9 +1473,10 @@ class ExceptionReportTestMixin:
 
     def verify_unsafe_email(self, view, check_for_POST_params=True):
         """
-        Asserts that potentially sensitive info are displayed in the email report.
+        Asserts that potentially sensitive info are displayed in the email
+        report.
         """
-        with self.settings(ADMINS=[("Admin", "admin@fattie-breakie.com")]):
+        with self.settings(ADMINS=["admin@example.com"]):
             mail.outbox = []  # Empty outbox
             request = self.rf.post("/some_url/", self.breakfast_data)
             if iscoroutinefunction(view):
@@ -1488,9 +1510,10 @@ class ExceptionReportTestMixin:
 
     def verify_safe_email(self, view, check_for_POST_params=True):
         """
-        Asserts that certain sensitive info are not displayed in the email report.
+        Asserts that certain sensitive info are not displayed in the email
+        report.
         """
-        with self.settings(ADMINS=[("Admin", "admin@fattie-breakie.com")]):
+        with self.settings(ADMINS=["admin@example.com"]):
             mail.outbox = []  # Empty outbox
             request = self.rf.post("/some_url/", self.breakfast_data)
             if iscoroutinefunction(view):
@@ -1531,9 +1554,10 @@ class ExceptionReportTestMixin:
 
     def verify_paranoid_email(self, view):
         """
-        Asserts that no variables or POST parameters are displayed in the email report.
+        Asserts that no variables or POST parameters are displayed in the email
+        report.
         """
-        with self.settings(ADMINS=[("Admin", "admin@fattie-breakie.com")]):
+        with self.settings(ADMINS=["admin@example.com"]):
             mail.outbox = []  # Empty outbox
             request = self.rf.post("/some_url/", self.breakfast_data)
             view(request)
